<?php
/**
 * Created by JetBrains PhpStorm.
 * User: erikasouza
 * Date: 03/01/13
 * Time: 17:45
 * To change this template use File | Settings | File Templates.
 */

namespace Estoque\Logic\ImplantacaoEstoque;

use \Estoque\Entities\Itlanctoest;
use \Estoque\Entities\Lanctoestoq;
use \Estoque\Logic\AtualizacaoEstoque\CalculaCustoLiquido;
use \Estoque\Logic\Exception\ExceptionLancamentoEstoque;
use \Estoque\Logic\AtualizacaoEstoque\AtualizadorDeEstoque;
use \Estoque\Logic\GeralLancamentoEstoque\LancamentoEstoque;
use \Estoque\Logic\FechamentoDia\FechamentoDia;
use \Estoque\Util\Parametros;

use \Geral\Entities\Almoxarifado;
use \Geral\Entities\Filial;
use \Geral\Entities\Localestoque;
use \Geral\Entities\Lote;
use \Geral\Entities\Retirada;
use \Geral\Entities\Prodfili;
use \Geral\Entities\Produto;
use \Geral\Util\Ambiente;
use \Geral\Util\DateUtil;
use \Geral\Util\NovoCodigo;

use \Planejamento\Entities\Cardadia;

use Setforms2\Runtime\Helper\Procedure\Param;
use Setforms2\Shared\Registry;

class Implantacao
{

    public static function preValidaImplantacao($nrorg, $filial, $dtLancto, $dtProc){
        try {
            LancamentoEstoque::validaData($dtLancto, Itlanctoest::TIPO_IMPLANTACAO, $dtProc);
            LancamentoEstoque::validaMesImplantacao($dtProc);

            $fechDia = new FechamentoDia($dtProc);
            $inventario = $fechDia->verificaInventarioPendente($nrorg, $filial, $dtProc);
            if ($inventario) {
                ExceptionLancamentoEstoque::erroDtIgualDtProcessamento();
            }

            self::validaOrdemImplantacao();
            $fechDia->validaInventarioRotativo($nrorg, $filial, $dtLancto);
        } catch (\Exception $e) {
            throw $e;
        }
    }


    public static function atualizaEstoqueImplantacao($filial, $nrLancto, $nrorg){
        $connection = Registry::getEntityManager();
        $connection->beginTransaction();

        try {
            /** @var $objLancto Lanctoestoq */
            $objLancto = Lanctoestoq::consultaLancamento($nrorg, $filial, $nrLancto);

            /** @var $objItens Itlanctoest */
            $objItens = Itlanctoest::consultaItens($filial, $nrLancto, $nrorg);
            if ($objItens) {
                /** @var $item Itlanctoest */
                foreach($objItens as $item) {
                    AtualizadorDeEstoque::atualizarEstoque($objLancto, $item);
                }
            } else {
                AtualizadorDeEstoque::atualizarEstoque($objLancto);
            }

            CalculaCustoLiquido::CalcularCustoLiquido($filial, $nrLancto, 0);


            $connection->flush();
            $connection->commit();

            return true;
        } catch(\Exception $e){
            $connection->rollback();
            throw $e;
        }
    }


    public static function validaVrUnitario($vrUniLiq, $vrUniBrut, $qtdItem){
        if ((float)$vrUniLiq < 0.00 AND (float)$vrUniBrut > 0.00) {
            ExceptionLancamentoEstoque::erroVrLiqMenorZero();
        }

        if ((float)$vrUniBrut < 0.00 AND (float)$vrUniLiq > 0.00) {
            ExceptionLancamentoEstoque::erroVrBrutoMenorZero();
        }

        if ((float)$qtdItem == 0.000 AND ((float)$vrUniLiq != 0.00 OR (float)$vrUniBrut != 0.00)) {
            ExceptionLancamentoEstoque::erroVrUniIgualZero();
        }

        if ((float)$vrUniLiq == 0.00) {
            return $vrUniLiq;
        }

        return $vrUniBrut;
    }


    public static function retornaCustoMedioProduto(
        $nrorg, $filial, $cdProdEsto, $almox, $local, $lote, $subLote, $dtImpl
    ){
        $dtProc = \Estoque\Util\Parametros::getDataProcessamento();
        $tipoCustoFilial = Parametros::getTipoCusto();
        switch ($tipoCustoFilial){
            case 8:
            case 9:
                $tipoCustoLiq = 8;
                $tipoCustoBruto = 9;
                break;
            case 3:
            case 4:
                $tipoCustoLiq = 3;
                $tipoCustoBruto = 4;
                break;
            default:
                $tipoCustoLiq = $tipoCustoBruto = (int)$tipoCustoFilial;
                break;

        }

        /** @var $objProdEsto Produto */
        $objProdEsto = Produto::getObject(array('cdproduto' => $cdProdEsto, 'nrorg' => $nrorg));
        $vrFatoConv = $objProdEsto->getVrfatoconv();

        /** @var $produtoRepository \Geral\Repositories\Produto */
        $produtoRepository = Produto::getRepository();
        $vrUnitarioLiq = $produtoRepository->executaCustoProduto(
            $filial, $cdProdEsto, $tipoCustoLiq, $almox, $local, $lote, $subLote, $dtImpl, $dtProc
        );
        $vrUnitarioLiq = $vrUnitarioLiq['P_RESULT'] * $vrFatoConv;

        $vrUnitarioBruto = $produtoRepository->executaCustoProduto(
            $filial, $cdProdEsto, $tipoCustoBruto, $almox, $local, $lote, $subLote, $dtImpl, $dtProc
        );
        $vrUnitarioBruto = $vrUnitarioBruto['P_RESULT'] * $vrFatoConv;

        return array($vrUnitarioLiq, $vrUnitarioBruto);
    }


    private static function excluiItemEvoluido($nrorg, $filial, $nrLancto){
        try {
            /** @var $objLancto Lanctoestoq */
            $objLancto = Lanctoestoq::consultaLancamento($nrorg, $filial, $nrLancto);
            $dtProc = DateUtil::getDataDeString(Parametros::getDataProcessamento());
            $dtImpl = $objLancto->getDtlancestq();

            $dtMaior = DateUtil::retornaMaiorData($dtProc, $dtImpl);
            $dtMenor = DateUtil::retornaMenorData($dtProc, $dtImpl);

            AtualizadorDeEstoque::tratamentoProdutosIrregulares($objLancto, $dtMaior, $dtMenor, true);

            return true;
        } catch (\Exception $e) {
            throw $e;
        }
    }


    public static function validaPreencAutoQtdProduto($nrorg, $filial, $nrLanc){
        /** @var $item array */
        $item = Itlanctoest::consultaItens($filial, $nrLanc, $nrorg);
        if (count($item) > 0) {
            return true;
        }

        return false;
    }


    public static function verificaAjuste(
        $filial, $cdProduto, $almox, $local, $lote, $subLote, $dtLanc, $ordem, $nrorg
    ){
        $itenLanc = new Itlanctoest();
        $itens = $itenLanc->verificaMovimentacaoEvoluida(
            $filial, $cdProduto, $almox, $local, $lote, $subLote,
            $dtLanc, $ordem, Itlanctoest::TIPO_AJUSTE, $nrorg
        );

        if ($itens) {
            if ($ordem === Itlanctoest::ORDEM_ULTIMO) {
                ExceptionLancamentoEstoque::erroProdAjustUltimo();
            } else {
                ExceptionLancamentoEstoque::erroProdAjustPrimeiro();
            }
        }

        return true;
    }


    public static function verificaAjusteLancamento($filial, $nrorg, $nrLancto, $dtImpl){
        $objItens = Itlanctoest::consultaItens($filial, $nrLancto, $nrorg);
        /** @var $item Itlanctoest */
        foreach($objItens as $item) {
            self::verificaAjuste(
                $filial, $item->getProduto()->getCdproduto(), $item->getLocalestoq()->getAlmoxarife()->getCdalmoxarife(),
                $item->getLocalestoq()->getCdlocalestoq(), $item->getNrloteestq(), $item->getNrsublote(),
                $dtImpl, $item->getIdordemclc(), $nrorg
            );
        }

        /** @var $objLancto Lanctoestoq */
        $objLancto = Lanctoestoq::consultaLancamento($nrorg, $filial, $nrLancto);
        AtualizadorDeEstoque::atualizarEstoque($objLancto);

        return true;
    }


    public static function validaOrdemImplantacao(){
        $ordem = Parametros::getOrdemImplantacao();
        switch($ordem){
            case null:
                \Geral\Util\AcumuladorDeMensagens::getInstance()->addMensagem('est42000_primeiroLanc', array('Implantação'));
                break;
            case "P":
                \Geral\Util\AcumuladorDeMensagens::getInstance()->addMensagem('est42000_primeiroLanc', array('Implantação'));
                break;
            case "U":
                \Geral\Util\AcumuladorDeMensagens::getInstance()->addMensagem('est42000_ultimoLanc', array('Implantação'));
                break;
            default:
                break;
        }

        return true;
    }


    public static function excluiItensImplantacao($nrorg, $filial, $nrLancto){
        $connect = Registry::getEntityManager();
        $connect->beginTransaction();

        try {
            $itens = Itlanctoest::consultaItens($filial, $nrLancto, $nrorg);
            foreach ($itens as $valorItem) {
                /** @var $valorItem Itlanctoest */
                if ($valorItem->getIdevolestoq() === 'S'){
                    self::excluiItemEvoluido($nrorg, $filial, $nrLancto);
                }

                $valorItem->delete();
                $connect->flush();
            }

            $connect->commit();
            return true;
        } catch(\Exception $e){
            $connect->rollback();
            throw $e;
        }

    }


    public static function verificaProdutoPorFilial($filial, $cdProduto, $nrorg){
        /** @var $objProdfili Prodfili */
        $objProdfili = Prodfili::getObject(
            array(
                'cdfilial' => $filial,
                'cdproduto' => $cdProduto,
                'nrorg' => $nrorg,
            )
        );

        if (!$objProdfili) {
            ExceptionLancamentoEstoque::erroProdNaoCad();
        } else if ($objProdfili->getIdprodativ() === 'I') {
            ExceptionLancamentoEstoque::erroProdInvalido();
        } else if ($objProdfili->getIdcntrestoq() === 'N') {
            ExceptionLancamentoEstoque::erroProdNaoCtrlEstoq();
        }

        return true;
    }


    public static function preValidaItem(
        $nrorg, $filial, $dtLancto, $produto, $vrLiq, &$vrBruto, $qtTotal, $ordem, &$almox, &$local, &$lote, &$subLote
    ){
        try {
            LancamentoEstoque::validaAlmoxLocalLoteSublote($almox, $local, $lote, $subLote);
            $vrBruto = Implantacao::validaVrUnitario($vrLiq, $vrBruto, $qtTotal);
            LancamentoEstoque::validaArredondamento($produto, $filial, $nrorg, $qtTotal);
            Implantacao::verificaProdutoPorFilial($filial, $produto, $nrorg);
         //   $lote = LancamentoEstoque::geraLote($filial, $nrorg, $subLote, $produto);
            Implantacao::verificaAjuste(
                $filial, $produto, $almox, $local, $lote, $subLote, $dtLancto, $ordem, $nrorg
            );

            return true;
        } catch (\Exception $e) {
            throw $e;
        }
    }


    public static function salvaItensImplantacao(
        $nrorg, $filial, $nrLancto, $dtLancto, $produto, $nmProd, $arvProd, $almox, $lote,
        $local, $subLote, $qtTotal, $vrTotal, $qtEstoq, $vrEstq, $seq, $vrLiq, $vrBruto, $ordem, $operAtual
    ){
        try{
            self::preValidaItem(
                $nrorg, $filial, $dtLancto, $produto, $vrLiq, $vrBruto,
                $qtTotal, $ordem, $almox, $local, $lote, $subLote
            );

            LancamentoEstoque::salvaItens(
                $nrorg, $filial, $nrLancto, $dtLancto, $seq, $produto, $qtTotal, $vrTotal, $ordem, $vrBruto, $vrLiq,
                $lote, $subLote, $local, $almox, $qtEstoq, $vrEstq, \Estoque\Entities\Itlanctoest::TIPO_IMPLANTACAO,
                $operAtual
            );

            $param = array($arvProd.' - '.$nmProd);
            \Geral\Util\AcumuladorDeMensagens::getInstance()->addMensagem('est41000_prodSalvo', $param);

            return true;
        } catch (\Exception $e) {
            $paramErro = ' | '.$arvProd.' - '.$nmProd;

            if ($e instanceof \Geral\Logic\Exception) {
                $getParam = $e->getParams();
                if (empty($getParam[0])) {
                    $e->setParams(array($paramErro));
                } else {
                    $e->addParam($paramErro);
                }
            }

            \Geral\Util\TratamentoDeMensagens::exibirMensagens($e);
        }
    }


}
